"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const anypoint_cli_command_1 = require("anypoint-cli-command");
const uris = require("../../../../helpers/uris");
const utils_1 = require("../../../../helpers/utils");
const fs = require("fs");
class Add extends anypoint_cli_command_1.AuthenticationBaseCommand {
    async run() {
        const { args, flags } = await this.parse(Add);
        const certificates = {
            publicKey: args.certificate,
            privateKey: args.privateKey,
            clientCertificate: flags.clientCertificate,
            crl: flags.crl,
        };
        await (0, utils_1.validateCertificates)(certificates.publicKey, certificates.privateKey, certificates.clientCertificate, certificates.crl);
        await this.doAdd(args.name, certificates, flags.verificationMode);
    }
    async doAdd(name, certificates, verificationMode) {
        var _a, _b;
        const orgId = this.session.selectedOrganization.id;
        const lb = await (0, utils_1.getLoadBalancerByName)(orgId, name);
        try {
            await this.req.patch(uris.loadBalancer(orgId, lb.vpc.id, lb.id), [{
                    op: 'add',
                    path: '/sslEndpoints/-',
                    value: {
                        publicKey: fs.readFileSync(certificates.publicKey, 'utf-8'),
                        privateKey: fs.readFileSync(certificates.privateKey, 'utf-8'),
                        ...certificates.clientCertificate && {
                            clientCert: fs.readFileSync(certificates.clientCertificate, 'utf-8'),
                            verifyClientMode: verificationMode || 'optional',
                        },
                        ...certificates.crl && { revocationList: fs.readFileSync(certificates.crl, 'utf-8') },
                        mappings: [{
                                inputUri: '/{app}/',
                                appName: '{app}',
                                appUri: '/',
                            }],
                    },
                }]);
            this.log(`Load balancer ${name} updated successfully`);
        }
        catch (error) {
            ((_b = (_a = error.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.message) ? this.error(error.response.data.message) : this.error(error);
        }
    }
}
exports.default = Add;
Add.description = 'Add an additional certificate to an existing Load balancer';
Add.flags = {
    ...anypoint_cli_command_1.AuthenticationBaseCommand.flags,
    help: core_1.Flags.help({ char: 'h' }),
    clientCertificate: core_1.Flags.string({ description: 'Client certificate file' }),
    verificationMode: core_1.Flags.string({
        description: 'Specifies the client verification mode',
        options: ['off', 'on', 'optional'],
    }),
    crl: core_1.Flags.string({ description: 'Certificate Revocation List file' }),
};
Add.args = [
    {
        name: 'name',
        required: true,
    },
    {
        name: 'certificate',
        required: true,
    },
    {
        name: 'privateKey',
        required: true,
    },
];
