"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const utils_1 = require("../../../../helpers/utils");
const uris = require("../../../../helpers/uris");
const anypoint_cli_command_1 = require("anypoint-cli-command");
const _ = require("lodash");
class Remove extends anypoint_cli_command_1.AuthenticationBaseCommand {
    constructor() {
        super(...arguments);
        this.lbSslEndpointRemove = async (name, certificateName) => {
            var _a, _b;
            const orgId = this.session.selectedOrganization.id;
            try {
                const lb = await (0, utils_1.getLoadBalancerByName)(orgId, name);
                if (certificateName === lb.sslEndpoints[lb.defaultSslEndpoint].publicKeyCN) {
                    this.error('The default SSL endpoint cannot be deleted!');
                }
                const index = _.findIndex(lb.sslEndpoints, { publicKeyCN: certificateName });
                if (index === -1) {
                    this.error('An SSL endpoint with Common Name ' + certificateName + ' was not found');
                }
                const body = [{
                        op: 'remove',
                        path: '/sslEndpoints/' + index,
                    }];
                await this.req.patch(uris.loadBalancer(orgId, lb.vpc.id, lb.id), body);
                this.log('Load balancer', lb.name, 'updated successfully');
            }
            catch (error) {
                ((_b = (_a = error === null || error === void 0 ? void 0 : error.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.message) ? this.error(error.response.data.message) : this.error(error);
            }
        };
    }
    async run() {
        const { args } = await this.parse(Remove);
        await this.lbSslEndpointRemove(args.name, args.certificateName);
    }
}
exports.default = Remove;
Remove.description = 'Remove a certificate from a Load balancer';
Remove.flags = {
    ...anypoint_cli_command_1.AuthenticationBaseCommand.flags,
    help: core_1.Flags.help({ char: 'h' }),
};
Remove.args = [{ name: 'name', required: true },
    { name: 'certificateName', required: true }];
