"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const utils = require("../../../helpers/utils");
const uris = require("../../../helpers/uris");
const anypoint_cli_command_1 = require("anypoint-cli-command");
class Delete extends anypoint_cli_command_1.AuthenticationBaseCommand {
    async run() {
        const { args } = await this.parse(Delete);
        await this.vpcDelete(args.name);
    }
    async vpcDelete(name) {
        var _a, _b;
        const session = this.session;
        const orgId = session.selectedOrganization.id;
        const vpc = await utils.getVpcByName(orgId, name);
        if (!vpc) {
            this.error(new Error(`VPC ${name} not found`));
        }
        try {
            const { status } = await this.req.delete(uris.vpc(orgId, vpc.id));
            if (status === 204) {
                this.log(`VPC ${name} deleted successfully`);
            }
            else {
                this.error(new Error(`Error deleting the VPC ${name}`));
            }
        }
        catch (error) {
            ((_b = (_a = error === null || error === void 0 ? void 0 : error.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.message) ? this.error(error.response.data.message) : this.error(error);
        }
    }
}
exports.default = Delete;
Delete.args = [
    {
        name: 'name',
        required: true,
    },
];
Delete.description = 'Delete an existing VPC';
Delete.flags = {
    ...anypoint_cli_command_1.AuthenticationBaseCommand.flags,
    help: core_1.Flags.help({ char: 'h' }),
    // flag with no value (-f, --force)
    force: core_1.Flags.boolean({ char: 'f' }),
};
