"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const utils = require("../../../../helpers/utils");
const uris = require("../../../../helpers/uris");
const anypoint_cli_command_1 = require("anypoint-cli-command");
const lodash_1 = require("lodash");
class Set extends anypoint_cli_command_1.AuthenticationBaseCommand {
    constructor() {
        super(...arguments);
        this.vpcDNSServersSet = async (name, server, domain) => {
            var _a, _b, _c;
            const orgId = this.session.selectedOrganization.id;
            try {
                const vpc = await utils.getVpcByName(orgId, name);
                const body = {
                    internalDns: {
                        specialDomains: [],
                        dnsServers: [],
                    },
                };
                body.internalDns.specialDomains = (0, lodash_1.isArray)(domain) ? domain : domain.split(',');
                body.internalDns.dnsServers = (0, lodash_1.isArray)(server) ? server : server.split(',');
                const response = await this.req.put(uris.vpc(orgId, vpc.id), body);
                if ((_a = response.data) === null || _a === void 0 ? void 0 : _a.erros) {
                    this.error(response.data.errors.map((error) => error.message).join(', '));
                }
                this.log('VPC DNS modified successfully');
            }
            catch (error) {
                ((_c = (_b = error === null || error === void 0 ? void 0 : error.response) === null || _b === void 0 ? void 0 : _b.data) === null || _c === void 0 ? void 0 : _c.message) ? this.error(error.response.data.message) : this.error(error);
            }
        };
    }
    async run() {
        const { args, flags } = await this.parse(Set);
        await this.vpcDNSServersSet(args.name, flags.server, flags.domain);
    }
}
exports.default = Set;
Set.description = 'Sets the domain names that are resolved using your internal DNS servers';
Set.flags = {
    ...anypoint_cli_command_1.AuthenticationBaseCommand.flags,
    help: core_1.Flags.help({ char: 'h' }),
    server: core_1.Flags.string({ description: 'IP for a DNS server to resolve special domains on', required: true }),
    domain: core_1.Flags.string({ description: 'A domain to resolve on the special DNS server list', required: true }),
};
Set.args = [{ name: 'name', required: true }];
