"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const anypoint_cli_command_1 = require("anypoint-cli-command");
const core_1 = require("@oclif/core");
const _ = require("lodash");
const flags_1 = require("../../../helpers/flags");
const uris_1 = require("../../../helpers/uris");
const moment = require("moment");
const flags_2 = require("anypoint-cli-command/lib/core/flags");
class List extends anypoint_cli_command_1.AuthenticationBaseCommand {
    constructor() {
        super(...arguments);
        this.cloudhubAlertList = async (limit, offset) => {
            var _a, _b;
            try {
                const getData = (0, uris_1.cloudAlerts)(limit, offset);
                const alerts = await this.req.get(getData.url, { params: getData.params });
                const rawData = [];
                const tableData = [];
                const header = ['Name', 'ID', 'Type', 'Severity', 'Source', 'Source Type', 'Enabled', 'Updated'];
                rawData.push(header);
                tableData.push(header);
                _.each(alerts.data.data, function (alert) {
                    const alertData = [
                        alert.name,
                        alert.id,
                        alert.condition.type,
                        alert.severity,
                        alert.condition.resources.join('\n'),
                        alert.condition.resourceType,
                        alert.enabled ? 'Yes' : 'No',
                        moment.utc(alert.lastModified).fromNow(),
                    ];
                    rawData.push(alertData);
                    tableData.push(alertData);
                });
                await this.show(tableData, rawData);
            }
            catch (error) {
                ((_b = (_a = error.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.message) ? this.error(error.response.data.message) : this.error(error);
            }
        };
    }
    async run() {
        const { flags } = await this.parse(List);
        await this.cloudhubAlertList(flags.limit, flags.offset);
    }
}
exports.default = List;
List.description = 'Lists all cloud alerts in the environment';
List.flags = {
    ...anypoint_cli_command_1.AuthenticationBaseCommand.flags,
    help: core_1.Flags.help({ char: 'h' }),
    limit: flags_1.limit,
    offset: flags_1.offset,
    ...flags_2.OutputFlag,
};
