"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const anypoint_cli_command_1 = require("anypoint-cli-command");
const JSZip = require("jszip");
const _ = require("lodash");
const moment = require("moment");
const uris_1 = require("../../../helpers/uris");
const Path = require('upath');
const fse = require('fs-extra');
class DownloadLogs extends anypoint_cli_command_1.AuthenticationBaseCommand {
    constructor() {
        super(...arguments);
        this.downloadInstanceLogs = async (appName, instanceId) => {
            const { data: loglines } = await this.req.get((0, uris_1.cloudAppV2Logs)(appName, instanceId));
            return {
                instanceId: instanceId,
                logLines: loglines,
            };
        };
        this.getLatestDeployment = (deployments) => {
            return deployments.data[0];
        };
        this.downloadDeploymentsLogs = async (deployments, name) => {
            if (deployments.data.length === 0) {
                throw new Error('This app has no logs');
            }
            const latestDeployment = this.getLatestDeployment(deployments);
            return (Promise.all(latestDeployment.instances.map(async (i) => {
                this.log('Downloading log for instance', i.instanceId);
                return (this.downloadInstanceLogs(name, i.instanceId));
            })));
        };
        this.appDownloadLogs = async (name, directory) => {
            const cloudAppDep = (0, uris_1.cloudAppV2Deployments)(name);
            const { data: deployments } = await this.req.get(cloudAppDep.url, { params: cloudAppDep.params });
            const responses = await this.downloadDeploymentsLogs(deployments, name);
            const latestDeployment = this.getLatestDeployment(deployments);
            return this.zipLogs(responses, latestDeployment, name, directory);
        };
    }
    async run() {
        const { args } = await this.parse(DownloadLogs);
        await this.appDownloadLogs(args.name, args.directory);
    }
    zipLogs(responses, deployment, name, directory) {
        const zip = new JSZip();
        _.each(responses, function (resp) {
            if (resp.logLines) {
                zip.file(resp.instanceId + '.txt', resp.logLines);
            }
        });
        const date = moment.utc(deployment.createTime).format('YYYY_MM_DD');
        const zipPath = Path.join(directory, name + '-' + date + '.zip');
        this.log('Saving to', zipPath);
        return zip
            .generateAsync({
            type: 'nodebuffer',
            // platform: process.platform
        })
            .then(function (data) {
            fse.writeFileSync(zipPath, data);
        });
    }
}
exports.default = DownloadLogs;
DownloadLogs.description = 'Download application logs to specified directory';
DownloadLogs.flags = {
    ...anypoint_cli_command_1.AuthenticationBaseCommand.flags,
};
DownloadLogs.args = [{ name: 'name', required: true }, { name: 'directory', required: true }];
