"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateApplicationStateExt = exports.getSourceApp = exports.dropSystemFields = exports.cleanAppData = exports.downloadSourceZip = exports.parseAppCreds = exports.storeStreamingResp = exports.handleCommonCreateUpdate = exports.getAppStatus = exports.showApplication = exports.showCloudhub2Application = exports.getSpecs = exports.lbToggleDynamicIPs = exports.modifyVpcAssociations = exports.findLoadBalancer = exports.toggleDynamicIps = exports.validateCertificates = exports.showFirewall = exports.showAlertHistory = exports.showLb = exports.showVpc = exports.getResourcesByKey = exports.getEnvironmentsByName = exports.getVPCsTable = exports.lbAllowlistRemove = exports.lbAllowlistAdd = exports.getLoadBalancerByName = exports.getLoadBalancers = exports.getLoadBalancersInVpc = exports.getVpcByName = exports.getVpcs = void 0;
const anypoint_cli_command_1 = require("anypoint-cli-command");
const moment = require("moment");
const util = require("util");
const cloudhubUris = require("./uris");
const utils_1 = require("../utils");
const uris_1 = require("./uris");
const Path = require('path');
const os = require("os");
const fs = require('fs');
const fmt = require('util').format;
const _ = require('lodash');
const APPLICATION_JSON = 'application/json';
const getVpcs = async (orgId) => (await anypoint_cli_command_1.req.get(cloudhubUris.vpcs(orgId))).data.data;
exports.getVpcs = getVpcs;
const getVpcByName = async (orgId, name) => {
    try {
        const vpcs = await (0, exports.getVpcs)(orgId);
        const data = _.find(vpcs, { name: name });
        if (data) {
            return Promise.resolve(data);
        }
        return Promise.reject(new Error(`A VPC with the name ${name} was not found`));
    }
    catch {
        return Promise.reject(new Error(`A VPC with the name ${name} was not found`));
    }
};
exports.getVpcByName = getVpcByName;
const getLoadBalancersInVpc = async (orgId, vpc) => {
    const lbs = (await anypoint_cli_command_1.req.get(cloudhubUris.loadBalancers(orgId, vpc.id))).data.data;
    _.each(lbs, function (_, index) {
        lbs[index].vpc = vpc;
    });
    return lbs;
};
exports.getLoadBalancersInVpc = getLoadBalancersInVpc;
const getLoadBalancers = async (orgId) => {
    const vpcs = await (0, exports.getVpcs)(orgId);
    return (await Promise.all(vpcs.map(vpc => (0, exports.getLoadBalancersInVpc)(orgId, vpc)))).flat();
};
exports.getLoadBalancers = getLoadBalancers;
const getLoadBalancerByName = async (orgId, name) => {
    const lbs = await (0, exports.getLoadBalancers)(orgId);
    return _.find(lbs, { name: name }) || Promise.reject(new Error(`A load balancer with the name ${name} was not found`));
};
exports.getLoadBalancerByName = getLoadBalancerByName;
const lbAllowlistAdd = async (self, name, cidrBlock) => {
    var _a, _b;
    const orgId = self.session.selectedOrganization.id;
    try {
        const lb = await (0, exports.getLoadBalancerByName)(orgId, name);
        const body = [{
                op: 'add',
                path: '/ipAllowlist/-',
                value: cidrBlock,
            }];
        await self.req.patch((0, uris_1.loadBalancer)(orgId, lb.vpc.id, lb.id), body);
        self.log(`Allowlist for Load balancer ${lb.name} updated successfully`);
    }
    catch (error) {
        ((_b = (_a = error === null || error === void 0 ? void 0 : error.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.message) ? self.error(error.response.data.message) : self.error(error);
    }
};
exports.lbAllowlistAdd = lbAllowlistAdd;
const lbAllowlistRemove = async (self, name, cidrBlock) => {
    var _a, _b;
    const orgId = self.session.selectedOrganization.id;
    try {
        const lb = await (0, exports.getLoadBalancerByName)(orgId, name);
        const index = lb.ipAllowlist.indexOf(cidrBlock);
        if (index === -1) {
            self.error(`No allowlist rule matching ${cidrBlock} was found`);
        }
        const body = [{
                op: 'remove',
                path: '/ipAllowlist/' + index,
            }];
        await self.req.patch((0, uris_1.loadBalancer)(orgId, lb.vpc.id, lb.id), body);
        self.log(`Allowlist for Load balancer ${lb.name} updated successfully`);
    }
    catch (error) {
        ((_b = (_a = error === null || error === void 0 ? void 0 : error.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.message) ? self.error(error.response.data.message) : self.error(error);
    }
};
exports.lbAllowlistRemove = lbAllowlistRemove;
function getEnvs(envs, associatedEnvs) {
    const retEnvs = [];
    for (const env of envs) {
        if (associatedEnvs.includes(env.id)) {
            retEnvs.push(env.name);
        }
    }
    return retEnvs;
}
function getOrgs(orgs, sharedWith) {
    const retOrgs = [];
    for (const org of orgs) {
        if (sharedWith.includes(org.id)) {
            retOrgs.push(org.name);
        }
    }
    return retOrgs;
}
async function getResource(self, selectedOrganization, resource) {
    switch (resource) {
        case 'environments': {
            const envs = await self.req.get(anypoint_cli_command_1.uris.environments(selectedOrganization));
            return envs.data;
        }
        case 'organizations': {
            const me = await self.req.get(anypoint_cli_command_1.uris.userMe);
            return me.data.memberOfOrganizations;
        }
        default:
            throw new Error('resource must be \'environments\' or \'organizations\'');
    }
}
async function getVPCsTable(self, vpcs, selectedOrganization) {
    const tableData = [];
    const environments = await getResource(self, selectedOrganization, 'environments');
    const organizations = await getResource(self, selectedOrganization, 'organizations');
    const header = ['Name', 'Region', 'CIDR Block', 'Inherited', 'Default', 'Environments', 'Business Groups'];
    tableData.push(header);
    for (const vpc of vpcs) {
        const data = [
            vpc.name,
            vpc.region,
            vpc.cidrBlock,
            vpc.ownerId === selectedOrganization ? 'No' : 'Yes',
            vpc.isDefault ? 'Yes' : 'No',
            getEnvs(environments.data, vpc.associatedEnvironments).join(', '),
            getOrgs(organizations, vpc.sharedWith).join(', '),
        ];
        tableData.push(data);
    }
    return tableData;
}
exports.getVPCsTable = getVPCsTable;
const getEnvironmentsByName = async (orgId, names) => {
    const environments = (await anypoint_cli_command_1.req.get(anypoint_cli_command_1.uris.environments(orgId))).data;
    return environments.data.filter(env => names.includes(env.name));
};
exports.getEnvironmentsByName = getEnvironmentsByName;
const getResourcesByKey = async (self, resource, value, key, args) => {
    const data = await getResource(self, self.session.selectedOrganization.id, resource);
    const parsedData = resource === 'environments' ? data.data : data;
    return _(parsedData)
        .keyBy(key)
        .at(args || [])
        .map(value)
        .value();
};
exports.getResourcesByKey = getResourcesByKey;
const showVpc = async (vpc, self) => {
    var _a, _b, _c, _d, _e;
    const table = [];
    table.push(['Name', vpc.name], ['Region', vpc.region], ['CIDR Block', vpc.cidrBlock], ['Inherited', vpc.ownerId === self.session.selectedOrganization.id ? 'No' : 'Yes'], ['Organization default', vpc.isDefault ? 'Yes' : 'No'], ['Special domains', (_b = (_a = vpc.internalDns) === null || _a === void 0 ? void 0 : _a.specialDomains) === null || _b === void 0 ? void 0 : _b.join(', ')], ['DNS Servers', (_d = (_c = vpc.internalDns) === null || _c === void 0 ? void 0 : _c.dnsServers) === null || _d === void 0 ? void 0 : _d.join(', ')], ['Firewall rules', (_e = vpc.firewallRules) === null || _e === void 0 ? void 0 : _e.length]);
    const [environments, organizations, loadBalancers] = await Promise.all([
        (0, exports.getResourcesByKey)(self, 'environments', 'name', 'id', vpc.associatedEnvironments),
        (0, exports.getResourcesByKey)(self, 'organizations', 'name', 'id', vpc.sharedWith),
        self.req.get(cloudhubUris.loadBalancers(self.session.selectedOrganization.id, vpc.id))
            .then(function (lbs) {
            return _.map(lbs.data, 'name');
        }),
    ]);
    table.push(['Environments', environments.join(', ')], ['Business groups', organizations.join(', ')], ['Load balancers', loadBalancers.join(', ')]);
    return table;
};
exports.showVpc = showVpc;
const showLb = async (lb) => {
    const table = [];
    table.push(['Name', lb.name], ['Domain', lb.domain], ['State', lb.state], ['VPC', lb.vpc.name]);
    const ips = lb.ipAddresses || [];
    table.push(['IPs', ips.join(', ')], ['TLSv1', lb.tlsv1 ? 'Enabled' : 'Disabled']);
    if (lb.sslEndpoints.length === 1) {
        const firstEndpoint = lb.sslEndpoints[0];
        table.push(['Common Name', firstEndpoint.publicKeyCN]);
        if (firstEndpoint.publicKeySANs) {
            table.push(['Subject Alternate Names', firstEndpoint.publicKeySANs.join(', ')]);
        }
        if (firstEndpoint.clientCertCN) {
            table.push(['Client Certificate', firstEndpoint.clientCertCN], ['Client verification', _.capitalize(firstEndpoint.verifyClientMode)]);
        }
        if (firstEndpoint.revocationListDigest) {
            table.push(['Revocation list serial', firstEndpoint.revocationListSerial], ['CRL last update', firstEndpoint.revocationListLastUpdate], ['CRL next update', firstEndpoint.revocationListNextUpdate]);
        }
        table.push(['Proxy mappings', firstEndpoint.mappings.length]);
    }
    else {
        const defaultSslEndpoint = lb.sslEndpoints.splice(lb.defaultSslEndpoint, 1);
        const defSans = defaultSslEndpoint[0].publicKeySANs || [];
        const primaryCert = fmt('%s (%s)', defaultSslEndpoint[0].publicKeyCN, defSans.join(', '));
        const additionalCerts = _.map(lb.sslEndpoints, function (endpoint) {
            const sans = endpoint.publicKeySANs || [];
            return fmt('%s (%s)', endpoint.publicKeyCN, sans.join(', '));
        });
        table.push(['Primary Certificate (SANs)', primaryCert], ['Additional Certificates (SANs)', additionalCerts.join('\n')]);
    }
    table.push(['Allowlisted IPs', lb.ipAllowlist.join(', ')]);
    return table;
};
exports.showLb = showLb;
const showAlertHistory = async (self, history) => {
    const table = [];
    if (history.total === 0) {
        self.log('This alert has no history');
    }
    for (const entry of history.data) {
        table.push([moment.utc(entry.triggeredAt).format('llll'), 'trigger', util.inspect(entry.context)]);
    }
    return table;
};
exports.showAlertHistory = showAlertHistory;
const showFirewall = async (vpc) => {
    const table = [];
    table.push(['Index', 'CIDR Block', 'Protocol', 'From Port', 'To Port']);
    _.each(vpc.firewallRules, function (rule, index) {
        const data = [index,
            rule.cidrBlock,
            rule.protocol.toUpperCase(),
            rule.fromPort,
            rule.toPort === rule.fromPort ? '' : rule.toPort];
        table.push(data);
    });
    return table;
};
exports.showFirewall = showFirewall;
function validateCertificates(certificate, privateKey, clientCertificate, crl) {
    try {
        if (certificate) {
            fs.readFileSync(certificate);
        }
        if (privateKey) {
            fs.readFileSync(privateKey);
        }
        if (clientCertificate) {
            fs.readFileSync(clientCertificate);
        }
        if (crl) {
            fs.readFileSync(crl);
        }
        return Promise.resolve();
    }
    catch (error) {
        return Promise.reject(error.message);
    }
}
exports.validateCertificates = validateCertificates;
async function toggleDynamicIps(url, value) {
    const body = [{
            op: 'replace',
            path: '/staticIPsDisabled',
            value: value,
        }];
    await anypoint_cli_command_1.req.patch(url, body);
}
exports.toggleDynamicIps = toggleDynamicIps;
const findLoadBalancer = async (orgId, name, certificateName) => {
    const lb = await (0, exports.getLoadBalancerByName)(orgId, name);
    const index = certificateName ?
        _.findIndex(lb.sslEndpoints, { publicKeyCN: certificateName }) :
        lb.defaultSslEndpoint;
    if (index === -1) {
        return Promise.reject(new Error(`No certificate named ${certificateName} was found`));
    }
    return { lb, index };
};
exports.findLoadBalancer = findLoadBalancer;
const modifyVpcAssociations = async (self, action, type, option = false) => {
    if (!_.includes(['sharedWith', 'associatedEnvironments'], type)) {
        throw new Error('Type must be sharedWith or associatedEnvironments');
    }
    if (!_.includes(['add', 'remove'], action)) {
        throw new Error('Action must be \'add\' or \'remove\'');
    }
    const [vpc, ids] = await Promise.all([
        (0, exports.getVpcByName)(self.session.selectedOrganization.id, self.argv[0]),
        type === 'sharedWith' ?
            (0, exports.getResourcesByKey)(self, 'organizations', 'id', 'name', self.argv.slice(1, self.argv.length)) :
            (0, exports.getResourcesByKey)(self, 'environments', 'id', 'name', self.argv.slice(1, self.argv.length)),
    ]);
    const url = cloudhubUris.vpc(self.session.selectedOrganization.id, vpc.id);
    const body = {};
    body[type] = action === 'add' ?
        _.union(vpc[type], ids) :
        _.difference(vpc[type], ids);
    if ((type === 'associatedEnvironments') && option) {
        body.isDefault = action === 'add';
    }
    body[type] = body[type].filter(function (element) {
        return element !== undefined;
    });
    await anypoint_cli_command_1.req.put(url, body);
    self.log('VPC association modified successfully');
};
exports.modifyVpcAssociations = modifyVpcAssociations;
const lbToggleDynamicIPs = async (self, name, enable) => {
    var _a, _b;
    const orgId = self.session.selectedOrganization.id;
    try {
        const lb = await (0, exports.getLoadBalancerByName)(orgId, name);
        const url = cloudhubUris.loadBalancer(orgId, lb.vpc.id, lb.id);
        await toggleDynamicIps(url, enable);
        self.log('Load balancer', name, 'updated successfully');
    }
    catch (error) {
        ((_b = (_a = error.response) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.message) ? self.error(error.response.data.message) : self.error(error);
    }
};
exports.lbToggleDynamicIPs = lbToggleDynamicIPs;
const getSpecs = (app) => {
    let specs = [];
    const seenSpecs = [];
    const errs = [];
    for (const replica of app.replicas) {
        const dversion = replica.currentDeploymentVersion;
        if (seenSpecs.includes(dversion)) {
            continue;
        }
        let state = '';
        if (dversion === app.lastSuccessfulVersion) {
            state = ' (LAST SUCCESSFUL)';
            specs.push(dversion + state);
        }
        else if (dversion === app.desiredVersion) {
            state = ' (DESIRED)';
            specs.unshift(dversion + state);
        }
        seenSpecs.push(dversion);
        if (replica.reason !== '' && !errs.includes(replica.reason)) {
            errs.push(replica.reason);
        }
    }
    if (specs.length === 0) {
        specs = (app.desiredVersion) ? [app.desiredVersion + '  (DESIRED)'] : seenSpecs;
    }
    return { specs, errs };
};
exports.getSpecs = getSpecs;
function propertiesToArray(props) {
    const arr = [];
    for (const key in (props || {})) {
        if (key) {
            const value = props[key];
            arr.push(fmt('%s : %s', key, value));
        }
    }
    return arr;
}
const showCloudhub2Application = (app) => {
    const table = [];
    table.push(['Name', app.name], ['App Id', app.id], ['Status', (0, exports.getAppStatus)(app)], ['Updated', moment.utc(app.lastModifiedDate).fromNow()], ['Runtime', app.target.deploymentSettings.runtimeVersion.split(':')[0]], ['Target', app.target.targetId], ['Artifact Id', app.application.ref.artifactId], ['Replicas', app.target.replicas]);
    if (app.application.vCores) {
        table.push(['vCores per replica', app.application.vCores]);
    }
    else if (app.target.deploymentSettings.instanceType) {
        table.push(['Instance type', app.target.deploymentSettings.instanceType]);
    }
    table.push(['Update Strategy', app.target.deploymentSettings.updateStrategy]);
    const { specs, errs } = (0, exports.getSpecs)(app);
    table.push(['Specs', specs.join(',\n')]);
    if (app.target.deploymentSettings.http.inbound &&
        app.target.deploymentSettings.http.inbound.publicUrl) {
        table.push(['Public Endpoint', app.target.deploymentSettings.http.inbound.publicUrl]);
    }
    const propService = app.application.configuration['mule.agent.application.properties.service'];
    const props = propService.properties;
    table.push(['Properties', propertiesToArray(props).join(',\n')]);
    const secprops = propService.secureProperties;
    table.push(['Secure Properties', propertiesToArray(secprops).join(',\n')]);
    if (errs.length > 0) {
        table.push(['Desired config not applied, 1 or more replicas', errs.join(',\n')]);
    }
    return table;
};
exports.showCloudhub2Application = showCloudhub2Application;
const showApplication = (app) => {
    var _a, _b;
    const table = [];
    table.push(['Domain', app.fullDomain], ['Status', (0, exports.getAppStatus)(app)]);
    if (app.deploymentErrorMessage) {
        table.push(['Deployment error', app.deploymentErrorMessage]);
    }
    table.push(['Updated', moment.utc(app.lastUpdateTime).fromNow()], ['Runtime', (_a = app.muleVersion) === null || _a === void 0 ? void 0 : _a.version]);
    if (app.previousMuleVersion) {
        table.push(['Previous runtime', (_b = app.previousMuleVersion) === null || _b === void 0 ? void 0 : _b.version]);
    }
    table.push(['File name', app.fileName]);
    if (app.region) {
        table.push(['Region', app.region]);
    }
    table.push(['Persistent queues', app.persistentQueues], ['Persistent queues encrypted', app.persistentQueuesEncrypted], ['Static IPs enabled', app.staticIPsEnabled], ['Monitoring', app.monitoringAutoRestart ?
            'Enabled. Will auto restart when not responding' :
            'Disabled']);
    const workersString = `${app.workers.type.cpu}  *  ${app.workers.amount} 
  ${_.reduce(app.workerStatuses, (acc, ws) => `${acc} \nIP: ${ws.host}, ${ws.status}`, '')}`;
    table.push(['Workers', workersString]);
    const propertiesString = _.map(_.keys(app.properties), function (p) {
        return `${p} = ${app.properties[p]}`;
    }).join('\n');
    if (propertiesString.length > 0) {
        table.push(['Properties', propertiesString]);
    }
    return table;
};
exports.showApplication = showApplication;
const getAppStatus = (app) => {
    if (app.deploymentUpdateStatus === 'DEPLOY_FAILED' && app.status === 'STARTED') {
        return 'STARTED\n(Update failed)';
    }
    return app.deploymentUpdateStatus || app.status;
};
exports.getAppStatus = getAppStatus;
const handleCommonCreateUpdate = (flags, app) => {
    app = app || {};
    const appData = {};
    if (_.has(flags, 'runtime')) {
        _.set(appData, 'muleVersion.version', flags.runtime);
    }
    if (_.has(flags, 'region')) {
        appData.region = flags.region;
    }
    if (_.has(flags, 'workers')) {
        _.set(appData, 'workers.amount', flags.workers);
    }
    if (_.has(flags, 'workerSize')) {
        _.set(appData, 'workers.type.name', getWorkerSizeNameFromVCores(flags.workerSize));
    }
    if (_.has(flags, 'propertiesFile')) {
        appData.properties = (0, utils_1.parseFile)(flags.propertiesFile, utils_1.parseProperties);
    }
    else if (_.has(flags, 'property')) {
        appData.properties = (0, utils_1.parseProperties)(flags.property, app.properties);
    }
    if (_.has(flags, 'secureProperty')) {
        appData.secureProperties = (0, utils_1.parseProperties)(flags.secureProperty, app.secureProperties);
    }
    if (_.has(flags, 'persistentQueues')) {
        appData.persistentQueues = flags.persistentQueues;
    }
    if (_.has(flags, 'persistentQueuesEncrypted')) {
        appData.persistentQueuesEncrypted = flags.persistentQueuesEncrypted;
    }
    if (_.has(flags, 'staticIPsEnabled')) {
        appData.staticIPsEnabled = flags.staticIPsEnabled;
    }
    if (_.has(flags, 'objectStoreV1')) {
        appData.objectStoreV1 = flags.objectStoreV1;
    }
    if (_.has(flags, 'objectStoreV2')) {
        appData.objectStoreV1 = !flags.objectStoreV2;
    }
    if (_.has(flags, 'autoRestart')) {
        appData.monitoringEnabled = flags.autoRestart;
        appData.monitoringAutoRestart = flags.autoRestart;
    }
    return appData;
};
exports.handleCommonCreateUpdate = handleCommonCreateUpdate;
const getWorkerSizeNameFromVCores = (vCores) => {
    switch (vCores.toString()) {
        case '0.1':
            return 'Micro';
        case '0.2':
            return 'Small';
        case '1':
            return 'Medium';
        case '2':
            return 'Large';
        case '4':
            return 'xLarge';
        default:
            throw new Error('Invalid workerSize. Valid values are 0.1, 0.2, 1, 2, 4');
    }
};
const storeStreamingResp = (resp, fpath) => {
    return new Promise(resolve => {
        const wstream = fs.createWriteStream(fpath);
        resp.pipe(wstream);
        wstream.on('finish', () => {
            resolve(fpath);
        });
    });
};
exports.storeStreamingResp = storeStreamingResp;
const parseAppCreds = (raw) => {
    const creds = parseCreds(raw);
    if (creds.entity.includes('/')) {
        const pieces = splitByFirst(creds.entity, '/');
        creds.organization = pieces[0];
        creds.app = pieces[1];
    }
    else {
        creds.app = creds.entity;
    }
    delete creds.entity;
    return creds;
};
exports.parseAppCreds = parseAppCreds;
function parseCreds(raw) {
    const hasEnv = raw.includes(':');
    const numSlashes = countOccurs(raw, '/');
    if (!hasEnv && numSlashes <= 1) {
        return { entity: raw };
    }
    const creds = {};
    const pieces = splitByFirst(raw, '/');
    const orgEnv = pieces[0];
    creds.entity = pieces[1];
    if (hasEnv) {
        const orgEnvPieces = splitByFirst(orgEnv, ':');
        creds.organization = orgEnvPieces[0];
        creds.environment = orgEnvPieces[1];
    }
    else {
        creds.organization = orgEnv;
    }
    return creds;
}
function countOccurs(str, char) {
    return str.split(char).length - 1;
}
function splitByFirst(str, sep) {
    const ind = str.indexOf(sep);
    return [str.slice(0, ind), str.slice(ind + 1, str.length)];
}
const downloadSourceZip = (self, sourceSess, app) => {
    const sourceZipUrl = cloudhubUris.cloudAppFileDownload(app.domain, app.fileName);
    const tmpfile = Path.join(os.tmpdir(), app.fileName);
    return self.req.get(sourceZipUrl, { headers: { Accept: '*/*',
            'X-ANYPNT-ENV-ID': sourceSess.envId.id,
            'X-ANYPNT-ORG-ID': sourceSess.organization },
        responseType: 'stream' })
        .then(function (resp) {
        return anypoint_cli_command_1.utils.storeStreamingResp(resp.data, tmpfile);
    });
};
exports.downloadSourceZip = downloadSourceZip;
const cleanAppData = (sourceApp, targetOpts) => {
    let data = _.cloneDeep(sourceApp);
    data = (0, exports.dropSystemFields)(data);
    data.objectStoreV1 = !sourceApp.cloudObjectStoreRegion;
    data.domain = targetOpts.app;
    return data;
};
exports.cleanAppData = cleanAppData;
const dropSystemFields = (data) => {
    const systemFields = ['versionId',
        'fullDomain',
        'workerStatuses',
        'lastUpdateTime',
        'previousMuleVersion',
        'userName',
        'persistentQueuesEncryptionEnabled',
        'hasFile',
        'ipAddresses',
        'status'];
    _.each(systemFields, function (field) {
        delete data[field];
    });
    return data;
};
exports.dropSystemFields = dropSystemFields;
const getSourceApp = (self, sessionX) => {
    try {
        return self.req.get(cloudhubUris.cloudAppV2(sessionX.app), { headers: { 'X-ANYPNT-ENV-ID': sessionX.envId.id,
                'X-ANYPNT-ORG-ID': sessionX.organization } });
    }
    catch (error) {
        throw new Error(`Source app: ${error.message}`);
    }
};
exports.getSourceApp = getSourceApp;
const updateApplicationStateExt = async (self, orgId, envId, id, state) => {
    try {
        const reqData = {
            url: cloudhubUris.cloudHub2App(orgId, envId, id),
            body: { application: { desiredState: state } },
            headers: { 'Content-Type': APPLICATION_JSON },
        };
        await self.req.patch(reqData.url, reqData.body, reqData.headers);
        self.log('OK, ' + state);
    }
    catch (error) {
        if (!_.isEmpty(error.response) && error.response.status === 304) {
            self.error('Application already in this state');
        }
        else {
            self.error(`Changing app state with app id ${id} errored`);
        }
    }
};
exports.updateApplicationStateExt = updateApplicationStateExt;
